@@ -2,8 +2,6 @@
 
 """ The work time log app tracks working hours in an interaction model similar to git. """
 
-__version__: str = "0.99.2.post1"
-
 import csv
 import datetime as dt
 import os
@@ -13,7 +11,17 @@ from collections import defaultdict, namedtuple
 from dataclasses import dataclass
 from fnmatch import fnmatch
 from itertools import chain, cycle, islice
-from typing import Callable, DefaultDict, Dict, List, Optional, Set, Tuple, cast
+from typing import (
+    Callable,
+    Collection,
+    DefaultDict,
+    Dict,
+    List,
+    Optional,
+    Set,
+    Tuple,
+    cast,
+)
 
 from work_components import consts, dt_parse, migrate, util
 from work_components.arguments import (
@@ -21,10 +29,10 @@ from work_components.arguments import (
     MAINTENANCE_NAMES,
     START_NAME,
     STOP_NAME,
-    SWITCH_NAMES,
+    SWITCH_NAME,
     Arguments,
 )
-from work_components.consts import DIRECTORY, DIRECTORY_DEBUG
+from work_components.consts import DIRECTORY, DIRECTORY_DEBUG, FLAG_FILE, RC_FILE_PATH
 from work_components.container import (
     OverlapError,
     ProtocolDay,
@@ -33,10 +41,13 @@ from work_components.container import (
     ShadowProtocolDay,
     sort_and_merge,
 )
-from work_components.rc_dao import RC
-from work_components.recess_dao import RecessDao
+from work_components.dao.core import WorkDao
+from work_components.dao.flags import Flags
+from work_components.dao.rc import RC
+from work_components.dao.recess import RecessDao
 from work_components.util import Color, PrinTable
-from work_components.work_dao import WorkDao
+
+__version__: str = consts.VERSION
 
 assert sys.version_info >= (3, 8)
 
@@ -49,27 +60,36 @@ DATE_FORMAT_FULL = "%A, " + DATE_FORMAT
 class Work:
     """Main class"""
 
+    debug: bool = False
+
     def __init__(self):
+        """Initialize instance variables and load the RC."""
         self.base_dir: pathlib.Path
         self.dao: WorkDao
         self.recess_dao: RecessDao
         self.configuration: RC
+        self.flags: Flags
 
-    def _reinit(self, base_dir: pathlib.Path) -> None:
-        """Connect the DAO and load the RC"""
+        # Load and check configuration (may raise)
+        self.configuration = RC.load_rc()
 
-        # Connect the DAOs
+        # Load flags (no checks)
+        self.flags = Flags()
+
+    def _connect(self, base_dir: pathlib.Path) -> None:
+        """Connect the DAOs."""
         self.dao = WorkDao(base_dir)
         self.recess_dao = RecessDao(base_dir)
 
-        # Load and check configuration
-        self.configuration = RC.load_rc()
-
     def main(self) -> None:
         """Main program flow."""
 
+        # Aliases and macros
+        Arguments.inject_aliases(self.configuration.aliases)
+        Arguments.substitute_macros(self.configuration.macros)
+
         # Argument parsing
-        parser = Arguments.create_argparser(version=__version__, program=self)
+        parser = Arguments.create_argparser(program=self)
         args = parser.parse_args()
 
         if args.help_verbose:
@@ -84,29 +104,24 @@ class Work:
         if args.dry_run:
             print(">>> Dry run: Only output <<<")
 
-        try:
-            # Connect DAO with normal or debug dir and load RC file
-            self.base_dir = DIRECTORY if not args.debug else DIRECTORY_DEBUG
-            self._reinit(base_dir=self.base_dir)
+        # Allow external access to debug flag
+        Work.debug = args.debug
 
-            # Only verify the protocol and state for non-maintenance modes
-            if args.mode not in MAINTENANCE_NAMES:
-                self.run_checks()
+        # Connect DAO with normal or debug dir and load RC file
+        self.base_dir = DIRECTORY if not args.debug else DIRECTORY_DEBUG
+        self._connect(base_dir=self.base_dir)
 
-            args.func(args)
-        # Invalid operations are considered part of the program flow and printed
-        except InvalidOperationWarning as warn:
-            print(warn)
-            sys.exit(1)
-        # Exceptions are handled depending on the mode
-        except Exception as err:  # pylint: disable=broad-except
-            # Debug mode: Raise
-            if args.debug:
-                raise err
+        # Migrate RC file
+        self.configuration.migrate(self.flags)
 
-            # Normal mode: In case of an exception, print the message and exit.
-            print(f"{Color.color('ERROR', Color.RED)}: {err}")
-            sys.exit(1)
+        # Only verify the protocol and state for non-maintenance modes
+        if args.mode not in MAINTENANCE_NAMES:
+            self.run_checks()
+
+        # Print "what's new" message once for each release
+        migrate.print_whats_new_in(__version__, self.flags)
+
+        args.func(args)
 
     def run_checks(self) -> None:
         """
@@ -165,10 +180,10 @@ class Work:
     ### Argument parsing / Step two ###
 
     def get_single_time(self, args) -> dt.datetime:
-        """Resolve the single time argument (start, stop)."""
+        """Resolve the single time argument (start, stop, switch)."""
 
-        if args.mode not in [START_NAME, STOP_NAME]:
-            raise ValueError("Expecting start or stop mode!")
+        if args.mode not in [START_NAME, STOP_NAME, SWITCH_NAME]:
+            raise ValueError("Expecting start, stop, or switch mode!")
 
         time_arg = args.time
 
@@ -200,7 +215,7 @@ class Work:
     ) -> Tuple[dt.datetime, dt.datetime]:
         """Resolve the from and to time arguments."""
 
-        if args.mode not in [ADD_NAME, *SWITCH_NAMES]:
+        if args.mode not in [ADD_NAME, SWITCH_NAME]:
             raise ValueError("Expecting add or switch mode!")
 
         # Add and pause work in reverse: when adding / pausing with "now now" at 12:50, we
@@ -304,10 +319,10 @@ class Work:
             raise InvalidOperationWarning.cant(action, f"{time_desc} needs to be today")
 
         # Allows starting only a few minutes early
-        m: float = 10
-        if start_time > (dt.datetime.now() + dt.timedelta(minutes=m)):
+        ten: float = 10
+        if start_time > (dt.datetime.now() + dt.timedelta(minutes=ten)):
             raise InvalidOperationWarning.cant(
-                action, f"{time_desc} may not be more than {m} minutes in the future"
+                action, f"{time_desc} may not be more than {ten} minutes in the future"
             )
 
     def _start(
@@ -454,13 +469,12 @@ class Work:
         - switch A B -> stop at time A, start at B (e.g. after taking a break)
         - switch A -> stop at A and immediately start again (e.g. after switching tasks)
         """
-        migrate.warn_deprecated_switch_name(args.mode)
 
-        switch_time: dt.datetime = dt_parse.resolve_time_argument(
-            args.time_s,
-            dt.date.today(),
-            rounding_mode=dt_parse.RoundingMode.DOWN,
-        )
+        switch_time: dt.datetime = self.get_single_time(args)
+
+        # If --stop is specified, only stop
+        if args.stop:
+            return self.stop(args)
 
         # If not specified otherwise, restart immediately at switch time.
         restart_time: dt.datetime = switch_time
@@ -622,8 +636,11 @@ class Work:
                 f"\n> Last entry ends at {entries_today[-1].end.strftime('%H:%M')}"
             )
 
-        new_section = lambda s: f"\n\n{s}:"
-        new_section_line = lambda: "\n\t"
+        def new_section(title: str):
+            return f"\n\n{title}:"
+
+        def new_section_line():
+            return "\n\t"
 
         # How many hours are we expected to work?
         output += new_section("Workday status")
@@ -688,11 +705,13 @@ class Work:
 
         today: dt.date = dt.date.today()
         minutes_logged, minutes_active_run = self._total_minutes_worked()
-        new_section = lambda: "\n> "
+
+        def new_section():
+            return "\n> "
 
         header: str = "{} on record, {} active run".format(
             self._timedelta_str(minutes_logged),
-            self._timedelta_str(minutes_active_run),
+            self._timedelta_str(minutes_active_run) if self.dao.run_active() else "no",
         )
 
         minutes_to_work: float = self._minutes_to_work_today()
@@ -1215,7 +1234,7 @@ class Work:
         if active_start_to_include is not None:
             active_run_row: List[str] = prin_table.rows.pop()
             active_run_time_str: str = (
-                f"{active_start_to_include.strftime(TIME_FORMAT)} ~"
+                f"{active_start_to_include.strftime(TIME_FORMAT)} ~      "
             )
             active_run_row[0] = Color.color(active_run_time_str, Color.BLUE)
             prin_table.add_row(active_run_row)
@@ -1243,10 +1262,8 @@ class Work:
             none_found_msg += " on " + selected_days[0].strftime(DATE_FORMAT_FULL)
         # Continuous period (each two consecutive list elements are directly adjacent days)
         elif all(
-            [
-                selected_days[i - 1] == selected_days[i] - dt.timedelta(days=1)
-                for i in range(1, len(selected_days))
-            ]
+            selected_days[i - 1] == selected_days[i] - dt.timedelta(days=1)
+            for i in range(1, len(selected_days))
         ):
             none_found_msg += " for the period from {} to {}".format(
                 selected_days[0].strftime(DATE_FORMAT),
@@ -1270,19 +1287,6 @@ class Work:
             return "", 0
         return f" ~ {self._timedelta_str(break_length)} break", break_length
 
-    # day #
-
-    def day(self, args) -> None:
-        """List the current day, including the active run."""
-        # pylint: disable=unused-argument
-        self._list(
-            [dt.date.today()],
-            list_empty=True,
-            include_active=True,
-            print_breaks=True,
-            only_time=False,
-        )
-
     # view #
 
     def view(self, args) -> None:
@@ -1608,7 +1612,7 @@ class Work:
         selected_days: List[dt.date] = self.get_selected_period(args)
         writer = csv.writer(sys.stdout, lineterminator=os.linesep)
 
-        assert consts.PROTOCOL_VERSION == 3
+        assert consts.RECORDS_VERSION == 3
         writer.writerow(["start", "end", "category", "message"])
 
         date: dt.date
@@ -1704,8 +1708,8 @@ class Work:
                 baseline_date=old_val.date(),
                 rounding_mode=dt_parse.RoundingMode.NONE,
             )
-        except ValueError as e:
-            print(f"{Color.color('Parse error', Color.RED)}: {str(e)}")
+        except ValueError as val_err:
+            print(f"{Color.color('Parse error', Color.RED)}: {str(val_err)}")
             return self._new_time_or_not(time_name, old_val)
 
     def _new_text_field_or_not(self, field_name: str, old_val: str) -> str:
@@ -1816,6 +1820,8 @@ class Work:
         action: Optional[Callable] = None
         output: str = ""
 
+        # pylint: disable-msg=unnecessary-lambda-assignment
+
         if args.add_vacation:
             if len(args.add_vacation) not in [1, 2]:
                 raise ValueError("Expects either one or two arguments!")
@@ -1863,7 +1869,6 @@ class Work:
         """Add a vacation for the given period. May be either one or two arguments."""
         first: dt.date
         last: dt.date
-        action: Callable
 
         length: int = len(vacation_period)
         if length == 1:
@@ -1873,7 +1878,10 @@ class Work:
             raise InvalidOperationWarning.cant(
                 "add vacation", because="begin date lies after end date"
             )
-        action = lambda: self.recess_dao.add_vacation(first_day=first, last_day=last)
+
+        def action():
+            self.recess_dao.add_vacation(first_day=first, last_day=last)
+
         if length == 1:
             assert first == last
             return action, f"Added vacation on {first.strftime(DATE_FORMAT)}"
@@ -1888,18 +1896,34 @@ class Work:
         if not self.recess_dao.has_days(year=year):
             print(f"No free days stored for {year}.")
             return
+
         if holidays := self.recess_dao.get_holidays(year=year):
-            print("Holidays:")
-            for h in holidays:
-                print(f"  {h.date.strftime(DATE_FORMAT)}")
+            print(f"Holidays ({len(holidays)} days):")
+            for holi in holidays:
+                print(f"  {holi.date.strftime(DATE_FORMAT)}")
+            print()
+
         if reduced_hour_days := self.recess_dao.get_reduced_hour_days(year=year):
-            print("Reduced hour days:")
-            for r in reduced_hour_days:
-                print(f"  {r.date.strftime(DATE_FORMAT)} ({r.hours} hours)")
+            print(f"Reduced hour days ({len(reduced_hour_days)} days):")
+            for redu in reduced_hour_days:
+                print(f"  {redu.date.strftime(DATE_FORMAT)} ({redu.hours} hours)")
+            print()
+
         if vacations := self.recess_dao.get_vacations(year=year):
-            print("Vacation:")
-            for v in vacations:
-                print(f"  {v.date.strftime(DATE_FORMAT)}")
+            print(f"Vacations ({len(vacations)} days):")
+            for vaca_period in util.continuous_periods([v.date for v in vacations]):
+                formatted_string = "  "
+                postfix = ""
+                if len(vaca_period) > 1:
+                    formatted_string += vaca_period[0].strftime("%d.%m – ")
+                    postfix = f" ({len(vaca_period)} days)"
+                formatted_string += vaca_period[-1].strftime(DATE_FORMAT)
+                print(f"{formatted_string}{postfix}")
+            print()
+
+        print(
+            f"Total: {len(holidays + reduced_hour_days + vacations)} free days"  # type: ignore
+        )
 
     # rehash #
 
@@ -1926,23 +1950,15 @@ class Work:
         Formerly `rc()` and `see()`.
         """
 
-        if args.create:
-            try:
-                RC.create_rc_file()
-                return
-            except FileExistsError as fe_err:
-                raise InvalidOperationWarning.cant(
-                    "create RC file", "file already exists"
-                ) from fe_err
-        elif args.expected:
-            print(RC.example_rc_file())
+        if args.default:
+            print(RC.default_rc_content())
             return
         elif args.see:
             self._see(s_target=args.see)
             return
 
         # Default: Print configuration path
-        print(f"{RC.path()}")
+        print(f"{RC_FILE_PATH}")
 
     def _see(self, s_target) -> None:
         """See configuration details."""
@@ -1951,14 +1967,19 @@ class Work:
             print(
                 f"Records directory:   {self.dao.records_directory}\n"
                 f"Free days directory: {self.recess_dao.directory}\n"
-                f"Configuration file:  {RC.path()}"
+                f"Configuration file:  {RC_FILE_PATH}\n"
+                f"Flags file:          {FLAG_FILE}"
             )
-        elif s_target == "expected hours":
-            if self.configuration.expected_hours is None:
-                print("No expected hours configured.")
-            else:
-                for wd, eh in self.configuration.expected_hours.items():
-                    print(f"{wd:<10}: {eh}")
+        elif s_target == "expected-hours":
+            for weekday, expected in self.configuration.expected_hours.items():
+                print(f"{weekday:<10}: {expected}")
+        elif s_target == "aliases":
+            for mode, aliases in self.configuration.aliases.items():
+                for alias in aliases:
+                    print(f"{alias} = {mode}")
+        elif s_target == "macros":
+            for macro, expansion in self.configuration.macros.items():
+                print(f"{macro:<} = {expansion}")
         # When adding a new option, make sure to update the "choices" of the ArgumentParser!
         else:
             raise NotImplementedError(f"Target {s_target} unknown")
@@ -2001,7 +2022,7 @@ class Work:
     @staticmethod
     def _minutes_worked(records: List[Record]) -> float:
         """Return the summed up work times contained in the given records."""
-        return sum([r.get_minutes() for r in records])
+        return sum(r.get_minutes() for r in records)
 
     def _minutes_active_run(self) -> float:
         """Return the minutes the current run has been active. Raises if no run is active."""
@@ -2027,7 +2048,8 @@ class Work:
         minutes_protocol = self._minutes_logged_on(requested_date=requested_date)
         minutes_active_run: float = 0.0
         # FIXME: To prepare for multi-day runs, we should only count minutes that
-        # occurred during the requested date.
+        # occurred during the requested date. Currently, if the run was started
+        # yesterday, it would be discarded completely.
         active_start: Optional[dt.datetime] = self.dao.get_start_time()
         if active_start is not None and active_start.date() == requested_date:
             minutes_active_run = self._minutes_active_run()
@@ -2060,9 +2082,9 @@ class Work:
 
         pauses: float = 0.0
         last_end: dt.datetime = records[0].end
-        for r in records[1:]:
-            pauses += util.minutes_difference(start=last_end, end=r.start)
-            last_end = r.end
+        for rec in records[1:]:
+            pauses += util.minutes_difference(start=last_end, end=rec.start)
+            last_end = rec.end
 
         return pauses
 
@@ -2099,17 +2121,15 @@ class Work:
         return timedelta_str, sign
 
     def _expected_minutes(self) -> List[float]:
-        """Return a list of minutes to work on a regular week."""
-        normal_hours: List[float] = (
-            list(self.configuration.expected_hours.values())
-            if self.configuration.expected_hours
-            else [8.0] * 5 + [0.0] * 2
-        )
+        """Return a list of minutes to work on a regular week.
+
+        Does not consider recess days."""
+        normal_hours: Collection[float] = self.configuration.expected_hours.values()
         assert len(normal_hours) == 7
         return [hour * 60.0 for hour in normal_hours]
 
     def _minutes_per_days(self, days: List[dt.date]) -> List[Tuple[dt.date, float]]:
-        """Return a list of the minutes to work on the given days."""
+        """Return a list of the minutes to work on the given days. Considers recess days."""
 
         # This method is the single place where expected hours are calculated.
         # That means that we only need to consider vacations or holidays here.
@@ -2128,7 +2148,7 @@ class Work:
         return result
 
     def _minutes_per_day(self, day: dt.date) -> float:
-        """Return the minutes to work on the given day."""
+        """Return the minutes to work on the given day. Considers recess days."""
 
         mins_per_days = self._minutes_per_days(days=[day])[0]
         _, minutes = mins_per_days
@@ -2220,21 +2240,39 @@ class Work:
 
 
 class InvalidOperationWarning(Warning):
+    """Warn that an operation cannot be executed due to state or preconditions."""
+
+    # pylint: disable-msg=invalid-name
     @staticmethod
     def cant(do: str, because: str):
+        """Produce a warning that `do` is impossible due to `because`."""
         return InvalidOperationWarning(f"Can't {do} – {because}!")
 
 
 ### Main program trigger ###
 
 
-def main():
+def cli():
+    """Command line interface entrypoint."""
     try:
         Work().main()
     except KeyboardInterrupt:
         print("Cancelled")
         sys.exit(130)
+    # Invalid operations are considered part of the program flow and printed
+    except InvalidOperationWarning as warning:
+        print(warning)
+        sys.exit(1)
+    # Exceptions are handled depending on the mode
+    except Exception as error:  # pylint: disable=broad-except
+        # Debug mode: Raise
+        if Work.debug:
+            raise error
+
+        # Normal mode: In case of an exception, print the message and exit.
+        print(f"{Color.color('ERROR', Color.RED)}: {error}")
+        sys.exit(1)
 
 
 if __name__ == "__main__":
-    main()
+    cli()
