@@ -3,8 +3,9 @@
 
 import argparse
 import datetime as dt
+import sys
 import textwrap
-from typing import Dict, List, Optional, Union
+from typing import Dict, List, Optional
 
 from work_components import consts
 from work_components.util import Color
@@ -17,17 +18,16 @@ START_NAME =    "start"
 STOP_NAME =     "stop"
 CANCEL_NAME =   "cancel"
 RESUME_NAME =   "resume"
-SWITCH_NAMES = ["switch", "pause"]
+SWITCH_NAME =   "switch"
 ADD_NAME =      "add"
-STATUS_NAMES = ["status", "s"]
-HOURS_NAMES =  ["hours", "h"]
-LIST_NAMES =   ["list", "ls"]
-DAY_NAME =      "day"
+STATUS_NAME =   "status"
+HOURS_NAME =    "hours"
+LIST_NAME =     "list"
 VIEW_NAME =     "view"
 EXPORT_NAME =   "export"
-EDIT_NAMES =   ["edit", "e"]
-REMOVE_NAMES = ["remove", "rm"]
-RECESS_NAMES =   ["free-days", "recess"]
+EDIT_NAME =     "edit"
+REMOVE_NAME =   "remove"
+RECESS_NAME =   "free-days"
 
 CONFIG_NAME =   "config"
 REHASH_NAME =   "rehash"
@@ -40,15 +40,19 @@ class Mode:
 
     def __init__(
         self,
-        names: Union[str, List[str]],
+        name: str,
         help_text: Optional[str] = None,
         description: Optional[str] = None,
     ) -> None:
-        self.names = [names] if isinstance(names, str) else names
+        self.names = [name]
         self.help = help_text
         self.description = description or help_text
         self.parents: List[argparse.ArgumentParser] = []
 
+    def add_aliases(self, aliases: List[str]) -> None:
+        """Add alternative names ("aliases") to this mode."""
+        self.names.extend(aliases)
+
     def add_as_parser(self, subparsers) -> argparse.ArgumentParser:
         """
         Add this mode to the subparsers action as a new mode parser and
@@ -82,36 +86,29 @@ MODES: Dict[str, Mode] = {
     START_NAME: Mode(START_NAME, help_text="Start work"),
     STOP_NAME: Mode(STOP_NAME, help_text="Stop work"),
     ADD_NAME: Mode(ADD_NAME, help_text="Add a log entry"),
-    SWITCH_NAMES[0]: Mode(
-        SWITCH_NAMES,
+    SWITCH_NAME: Mode(
+        SWITCH_NAME,
         help_text="Short-hand for stop & start",
         description=(
-            "A short-hand for stop and start. Flags are passed to stop, meaning they "
-            "refer to the stopped run."
+            "Short-hand for stop & start. Can be used to log two adjacent entries "
+            "with a differing category or message. Category and message are passed "
+            "to stop, meaning they refer to the stopped run."
         ),
     ),
     CANCEL_NAME: Mode(CANCEL_NAME, help_text="Cancel the current run"),
     RESUME_NAME: Mode(
         RESUME_NAME, help_text='Resume the last run today (undo "work stop")'
     ),
-    STATUS_NAMES[0]: Mode(STATUS_NAMES, help_text="Print the current status"),
-    HOURS_NAMES[0]: Mode(HOURS_NAMES, help_text="Calculate hours worked"),
-    LIST_NAMES[0]: Mode(
-        LIST_NAMES,
+    STATUS_NAME: Mode(STATUS_NAME, help_text="Print the current status"),
+    HOURS_NAME: Mode(HOURS_NAME, help_text="Calculate hours worked"),
+    LIST_NAME: Mode(
+        LIST_NAME,
         help_text="List work records",
         description=(
             "List work records (by default the current day)."
             " For other ranges use the optional arguments."
         ),
     ),
-    DAY_NAME: Mode(
-        DAY_NAME,
-        help_text="List the current day with all details",
-        description=(
-            "List the current day with all details. Shorthand for "
-            '"list --include-active --with-breaks --list-empty"'
-        ),
-    ),
     VIEW_NAME: Mode(
         VIEW_NAME,
         help_text="Views on work records",
@@ -121,18 +118,18 @@ MODES: Dict[str, Mode] = {
         ),
     ),
     EXPORT_NAME: Mode(EXPORT_NAME, help_text="Export records as CSV"),
-    EDIT_NAMES[0]: Mode(
-        EDIT_NAMES,
+    EDIT_NAME: Mode(
+        EDIT_NAME,
         help_text="Edit work records",
         description="Edit work records (by default from the current day)",
     ),
-    REMOVE_NAMES[0]: Mode(
-        REMOVE_NAMES,
+    REMOVE_NAME: Mode(
+        REMOVE_NAME,
         help_text="Remove records from the log",
         description="Remove records from the log (by default from the current day)",
     ),
-    RECESS_NAMES[0]: Mode(
-        RECESS_NAMES,
+    RECESS_NAME: Mode(
+        RECESS_NAME,
         help_text="Manage free days (vacation, holidays, part-time days)",
         description="Manage free days (vacation, holidays, part-time days). Default mode: --list",
     ),
@@ -149,7 +146,34 @@ class Arguments:
     """Allows creating parsers or completions."""
 
     @staticmethod
-    def create_argparser(version, program) -> argparse.ArgumentParser:
+    def inject_aliases(alias_mapping: Dict[str, List[str]]) -> None:
+        """Inject aliases into modes. Ensures that the parser recognizes configured
+        aliases and that they are printed in the help output."""
+        for mode, aliases in alias_mapping.items():
+            MODES[mode].add_aliases(aliases)
+
+    @staticmethod
+    def substitute_macros(macros_mapping: Dict[str, str]) -> None:
+        """Replace any detected macros in `sys.argv` with the value(s) provided in
+        the given mapping."""
+        for macro, replacement in macros_mapping.items():
+            if macro not in sys.argv:
+                continue
+
+            macro_index = sys.argv.index(macro)
+            if any(
+                # Check if there is a valid mode argument before the macro
+                mode in sys.argv and sys.argv.index(mode) < macro_index
+                for mode in MODES
+            ):
+                # Do not raise. A macro name may overlap an argument name or value,
+                # in which case the macro is "found" when that value is passed.
+                continue
+
+            sys.argv[macro_index : macro_index + 1] = replacement.split(" ")
+
+    @staticmethod
+    def create_argparser(program) -> argparse.ArgumentParser:
         """Create an `ArgumentParser` instance and return it."""
 
         parser = argparse.ArgumentParser(
@@ -160,12 +184,14 @@ class Arguments:
         # fmt: off
         parser.add_argument("-H", "--help-verbose", action="store_true",
             help="show a longer help output, similar to a man page")
-        parser.add_argument("-V", "--version", action="version", version=f"%(prog)s {version}")
-        parser.add_argument("-d", "--debug", action="store_true", help="use a debug directory")
+        parser.add_argument("-V", "--version", action="version", version=f"%(prog)s {consts.VERSION}")
+        parser.add_argument("-d", "--debug", action="store_true", help=argparse.SUPPRESS)
         parser.add_argument("-y", "--dry-run", action="store_true", help="only print output")
         # fmt: on
 
-        modes = parser.add_subparsers(title="modes", dest="mode")
+        modes = parser.add_subparsers(
+            title="modes", dest="mode", metavar="{start, stop, add, status, list, ...}"
+        )
 
         # Shared help texts
         date_help_text = (
@@ -196,7 +222,7 @@ class Arguments:
 
         MODES[STOP_NAME].parents.append(category_message_parent)
         MODES[ADD_NAME].parents.append(category_message_parent)
-        MODES[SWITCH_NAMES[0]].parents.append(category_message_parent)
+        MODES[SWITCH_NAME].parents.append(category_message_parent)
 
         ## Parent parsers for dates ##
 
@@ -224,7 +250,7 @@ class Arguments:
                 metavar="DAY",
                 help=(
                     'A weekday or any matching prefix, e.g. "Mon", "su" or "wednesday". '
-                    "Always selects from the past seven days."
+                    "Always selects from the past seven days, excluding today."
                 ),
             )
         multi_date_sel_modes.add_argument(
@@ -260,11 +286,11 @@ class Arguments:
 
         # Single date selection
         MODES[ADD_NAME].parents.append(single_date_sel_parent)
-        MODES[EDIT_NAMES[0]].parents.append(single_date_sel_parent)
-        MODES[REMOVE_NAMES[0]].parents.append(single_date_sel_parent)
+        MODES[EDIT_NAME].parents.append(single_date_sel_parent)
+        MODES[REMOVE_NAME].parents.append(single_date_sel_parent)
 
         # Multi date selection
-        MODES[LIST_NAMES[0]].parents.append(multi_date_sel_parent)
+        MODES[LIST_NAME].parents.append(multi_date_sel_parent)
         MODES[VIEW_NAME].parents.append(multi_date_sel_parent)
         MODES[EXPORT_NAME].parents.append(multi_date_sel_parent)
 
@@ -307,19 +333,25 @@ class Arguments:
             ),
         )
 
-        switch_mode = MODES[SWITCH_NAMES[0]].add_as_parser(modes)
+        switch_mode = MODES[SWITCH_NAME].add_as_parser(modes)
         switch_mode.set_defaults(func=program.switch)
         switch_mode.add_argument(
-            "time_s",
+            "time",
             metavar="TIME",
             help=("The time to switch runs at. TIME: " + time_help_text),
         )
-        switch_mode.add_argument(
+        switch_modifier = switch_mode.add_mutually_exclusive_group()
+        switch_modifier.add_argument(
             "-s",
             "--start",
             metavar="TIME",
             help="Instead of restarting immediately, start the next run at TIME.",
         )
+        switch_modifier.add_argument(
+            "--stop",
+            action="store_true",
+            help="Do not restart after stopping.",
+        )
 
         # STATE dependent modes
 
@@ -334,21 +366,14 @@ class Arguments:
             help="Resume even if a run is active.",
         )
 
-        status_mode = MODES[STATUS_NAMES[0]].add_as_parser(modes)
+        status_mode = MODES[STATUS_NAME].add_as_parser(modes)
         status_mode.set_defaults(func=program.status)
         status_mode.add_argument(
             "-o", "--oneline", action="store_true", help="Print status in one line."
         )
 
-        hours_mode = MODES[HOURS_NAMES[0]].add_as_parser(modes)
+        hours_mode = MODES[HOURS_NAME].add_as_parser(modes)
         hours_mode.set_defaults(func=program.hours)
-        hours_mode.add_argument(
-            "-d",
-            "--workday",
-            action="store_true",
-            dest="h_workday",
-            help="Also show the end time of a complete workday with respect to the week balance.",
-        )
         hours_mode.add_argument(
             "-u",
             "--until",
@@ -372,8 +397,14 @@ class Arguments:
             action="store_const",
             dest="h_target",
             const="8",
-            help=target_arg_help_text_stub
-            + "an 8 hour day (equivalent to --target 8).",
+            help="Short-hand for --target 8.",
+        )
+        hours_mode.add_argument(
+            "-d",
+            "--workday",
+            action="store_true",
+            dest="h_workday",
+            help="Also show the end time of a complete workday with respect to the week balance.",
         )
         # Modifiers
         hours_mode.add_argument(
@@ -396,7 +427,7 @@ class Arguments:
 
         # PROTOCOL interaction modes
 
-        list_mode = MODES[LIST_NAMES[0]].add_as_parser(modes)
+        list_mode = MODES[LIST_NAME].add_as_parser(modes)
         list_mode.set_defaults(func=program.list_entries)
         list_mode.add_argument(
             "-e", "--list-empty", action="store_true", help="Include empty days."
@@ -417,9 +448,6 @@ class Arguments:
             help="Only show record times and omit all optional record attributes.",
         )
 
-        day_mode = MODES[DAY_NAME].add_as_parser(modes)
-        day_mode.set_defaults(func=program.day)
-
         view_mode = MODES[VIEW_NAME].add_as_parser(modes)
         view_mode.set_defaults(func=program.view)
         view_mode.add_argument(
@@ -451,15 +479,15 @@ class Arguments:
         export_mode = MODES[EXPORT_NAME].add_as_parser(modes)
         export_mode.set_defaults(func=program.export)
 
-        edit_mode = MODES[EDIT_NAMES[0]].add_as_parser(modes)
+        edit_mode = MODES[EDIT_NAME].add_as_parser(modes)
         edit_mode.set_defaults(func=program.edit)
 
-        remove_mode = MODES[REMOVE_NAMES[0]].add_as_parser(modes)
+        remove_mode = MODES[REMOVE_NAME].add_as_parser(modes)
         remove_mode.set_defaults(func=program.remove)
 
         # RECESS management
 
-        recess_mode = MODES[RECESS_NAMES[0]].add_as_parser(modes)
+        recess_mode = MODES[RECESS_NAME].add_as_parser(modes)
         recess_mode.set_defaults(func=program.recess)
         recess_mode_modes = recess_mode.add_mutually_exclusive_group()
         recess_mode_modes.add_argument(
@@ -472,7 +500,7 @@ class Arguments:
         recess_mode_modes.add_argument(
             "--add-holiday",
             metavar="DATE",
-            help="Add a holiday. DATE: " + date_help_text,
+            help="Add a holiday on DATE (see DATE).",
         )
         recess_mode_modes.add_argument(
             "--add-reduced-day",
@@ -507,21 +535,13 @@ class Arguments:
             help="Default mode: Print the path of the runtime configuration file.",
         )
         config_mode_modes.add_argument(
-            "-c",
-            "--create",
+            "--default",
             action="store_true",
-            help="Create a default runtime configuration with all currently active options.",
+            help="Print the content of the default runtime configuration.",
         )
         config_mode_modes.add_argument(
-            "-e",
-            "--expected",
-            action="store_true",
-            help="Print the contents of an expected runtime configuration file.",
-        )
-        config_mode_modes.add_argument(
-            "-s",
             "--see",
-            choices=["dir", "expected hours"],
+            choices=["dir", "expected-hours", "aliases", "macros"],
             help="Check how work is configured (see --help for options).",
         )
 
@@ -537,52 +557,59 @@ class Arguments:
         parser.print_help()
 
         two_spaces: str = " " * 2
-        fill = lambda s: textwrap.fill(
-            s,
-            width=80,
-            initial_indent=two_spaces,
-            subsequent_indent=two_spaces,
-            replace_whitespace=False,
-            drop_whitespace=False,
-        )
-        new_epilog = lambda t, c: "\n\n" + t + ":\n" + "\n".join(map(fill, c))
+
+        def fill(text: str):
+            return textwrap.fill(
+                text,
+                width=80,
+                initial_indent=two_spaces,
+                subsequent_indent=two_spaces,
+                replace_whitespace=False,
+                drop_whitespace=False,
+            )
+
+        def new_epilog(title: str, content: List[str]):
+            return "\n\n" + title + ":\n" + "\n".join(map(fill, content))
 
         print("\n\n" + Color.bold("EXTENDED HELP"), end="")
 
         # fmt: off
         print(
             new_epilog(
-                t="rounding",
-                c=[
+                title="rounding",
+                content=[
                     "Times are rounded in your favor, to the next full 15 minutes, when"
                         ' you enter "now" instead of an exact time. For example:',
                     '- "work start now" at 10:14 starts at 10:00',
                     '- "work stop now" at 19:01 stops at 19:15',
+                    'To override the rounding mode, use the keywords "now+", "now-", or'
+                        '"now!", which coerce rounding up, down, or prevent rounding, '
+                        "respectively."
                 ],
             )
             + new_epilog(
-                t="expected hours",
-                c=[
+                title="expected hours",
+                content=[
                     "For some sub-functions (mainly status and hours), you will be nudged to work "
                         "no more, but also no less, than the expected hours. By default, work will "
                         "expect 8 hours for every workday (Mon–Fri). Change the default by editing "
                         "the run-time configuration. To define individual days where less hours "
                         "should be expected, use the free-days command:",
-                    "- free-days --add-vacation for vacations (expect 0 hours)",
-                    '- free-days --add-holiday for public holidays (expect 0 hours)"',
-                    '- free-days --add-reduced-day for part-time days (expect < 8 hours)"',
+                    '- "free-days --add-vacation" for vacations (expect 0 hours)',
+                    '- "free-days --add-holiday" for public holidays (expect 0 hours)',
+                    '- "free-days --add-reduced-day" for part-time days (expect < 8 hours)',
                 ],
             )
             + new_epilog(
-                t="balance",
-                c=[
+                title="balance",
+                content=[
                     "Based on the expected hours and the time worked, two balances are "
                         "calculated:",
-                    "1) Week balance (shown in status): Total over-/undertime accumulated over "
+                    '1) Week balance (shown in "status"): Total over-/undertime accumulated over '
                         "the current week (starting Monday), up to the day before today.",
-                    "2) Current balance (shown in hours): This shows the hours you need to work "
+                    '2) Current balance (shown in "hours"): This shows the hours you need to work '
                         "on the current day and how many are remaining.",
-                    'To check your balance development over other periods, use "view balance."'
+                    'To check your balance development over other periods, use "view balance".'
                 ],
             )
         )
