@@ -6,7 +6,8 @@ import os
 import pathlib
 from typing import List, Tuple
 
-PROTOCOL_VERSION = 3
+VERSION: str = "0.100.1"
+RECORDS_VERSION: int = 3
 
 # Directories
 PARENT_DIR: pathlib.Path = pathlib.Path("~", ".local", "share").expanduser()
@@ -21,8 +22,22 @@ RUN_FILE_NAME = "running.wtime"
 RECESS_DIRECTORY_NAME = "recess"
 RECESS_FILE_EXTENSION = "wvac"
 
-# Configuration – user folder
-RC_FILE_PATH = os.path.expanduser("~/.workrc")
+
+def _resolve_xdg_path(env_var: str, default: str) -> str:
+    """Get path at `env_var` or default, and expand user and variables."""
+    path_or_default: str = os.environ.get(env_var, default)
+    return os.path.expanduser(os.path.expandvars(path_or_default))
+
+
+# Configuration – XDG_CONFIG_HOME
+_CONFIG_HOME: str = _resolve_xdg_path("XDG_CONFIG_HOME", os.path.join("~", ".config"))
+RC_FILE_PATH: str = os.path.join(_CONFIG_HOME, "workrc")
+
+# Flags – XDG_STATE_HOME
+_STATE_HOME: str = _resolve_xdg_path(
+    "XDG_STATE_HOME", os.path.join("~", ".local", "state")
+)
+FLAG_FILE: pathlib.Path = pathlib.Path(_STATE_HOME, "work", "flags.wstate")
 
 # Formats
 DATE_FORMAT = "%Y-%m-%d"
@@ -34,6 +49,7 @@ INFO_FILE_CONTENT = "work-time-protocol/protocol-v{}/last-edit:{}/checksum:{}\n"
 INFO_FILE_PATTERN = INFO_FILE_CONTENT.format(
     r"(\d{1,2})", r"(\d{4}-\d{2}-\d{2}\s\d{2}:\d{2})", r"(\d+)"
 )
+
 TIME_PATTERN = r"\d{4}-\d{2}-\d{2}\s\d{2}:\d{2}"
 YEAR_DIR_PATTERN = r"\d{4}"
 MONTH_DIR_PATTERN = r"(0[1-9]|1[0-2])"
