@@ -265,7 +265,7 @@ class ProtocolDay(ProtocolMeta):
 
         # "If csvfile is a file object, it should be opened with newline=''."
         # See: https://docs.python.org/3/library/csv.html
-        with open(self.path, "r", newline="") as protocol_file:
+        with open(self.path, "r", newline="", encoding="utf-8") as protocol_file:
             reader = csv.reader(protocol_file)
             for row in reader:
                 try:
@@ -414,7 +414,7 @@ class ProtocolDay(ProtocolMeta):
 
         # "If csvfile is a file object, it should be opened with newline=''."
         # See: https://docs.python.org/3/library/csv.html
-        with open(self.path, "w", newline="") as protocol_file:
+        with open(self.path, "w", newline="", encoding="utf-8") as protocol_file:
             protocol_writer = csv.writer(protocol_file, lineterminator=os.linesep)
             for record in records:
                 row: List[str] = record.to_protocol_row()
