@@ -4,6 +4,7 @@
 
 import datetime as dt
 import re
+import textwrap
 from collections import Counter
 from typing import Dict, List, Optional
 
@@ -49,10 +50,12 @@ def get_period(period_start: dt.date, period_end: dt.date) -> List[dt.date]:
     """
     Return a period defined by two dates.
 
-    The order of start and end does not influence the result.
+    Raises if `period_start` > `period_end`.
     """
+    if period_start > period_end:
+        raise ValueError("Period start lies after period end.")
 
-    period_ends: List[dt.date] = sorted([period_start, period_end])
+    period_ends: List[dt.date] = [period_start, period_end]
     start_day, end_day = period_ends
 
     period: List[dt.date] = []
@@ -64,17 +67,58 @@ def get_period(period_start: dt.date, period_end: dt.date) -> List[dt.date]:
     return period
 
 
+def adjacent_dates(left: dt.date, right: dt.date) -> bool:
+    """Check if the given dates are directly adjacent."""
+    return left + dt.timedelta(days=1) == right
+
+
 def is_continuous_period(period: List[dt.date]) -> bool:
     """Check if the period given is uninterrupted. Does not sort input list!"""
     if len(period) <= 1:
         return True
 
     for i in range(1, len(period)):
-        if period[i - 1] + dt.timedelta(days=1) != period[i]:
+        if not adjacent_dates(period[i - 1], period[i]):
             return False
     return True
 
 
+def continuous_periods(dates: List[dt.date]) -> List[List[dt.date]]:
+    """Split up the list of dates into lists of continuous periods contained within.
+
+    Important: Does not sort input list!
+
+    Returns a list of lists of dates."""
+    if len(dates) <= 1:
+        return [dates]
+
+    periods: List[List[dt.date]] = [dates[:1]]
+
+    for date in dates[1:]:
+        if not adjacent_dates(periods[-1][-1], date):
+            periods.append([date])
+            continue
+        periods[-1].append(date)
+
+    return periods
+
+
+def wrap_and_indent(*paragraphs: str, width=80, prefix="") -> str:
+    """Wrap and indent the given paragraph(s) and return the joined strings."""
+    result: List[str] = []
+    for paragraph in paragraphs:
+        result.append(
+            textwrap.fill(
+                paragraph,
+                width=width,
+                initial_indent=prefix,
+                subsequent_indent=prefix,
+                replace_whitespace=False,
+            )
+        )
+    return "\n".join(result)
+
+
 class Color:
     """See https://en.wikipedia.org/wiki/ANSI_escape_code#8-bit"""
 
